
#include "Serial2Wifi_SPI_Client.h"

Serial2Wifi_SPI_Client::Serial2Wifi_SPI_Client()
{
	this->writeTimeout = 50;
	this->readTimeout  = 50;
	this->initialized  = false;
	this->first_run    = true;
}



void Serial2Wifi_SPI_Client::initializeQuietly(int baudRate)
{
	if (this->initialized == true)
		this->close();

	char send_data[128];
	char read_data[128];

	if( !bcm2835_init() )
		throw "bcm2835 initialization failed";

	this->data_ready_pin = DATA_READY_PIN_1;
	bcm2835_gpio_fsel( this->data_ready_pin, BCM2835_GPIO_FSEL_INPT );

	this->manual_cs = MANUAL_SPI_CS_1;
	bcm2835_gpio_fsel( this->manual_cs, BCM2835_GPIO_FSEL_OUTP );
	bcm2835_gpio_set( this->manual_cs );

	bcm2835_spi_begin();
	bcm2835_spi_setBitOrder( BCM2835_SPI_BIT_ORDER_LSBFIRST );
	bcm2835_spi_setDataMode( BCM2835_SPI_MODE0 );
	bcm2835_spi_setClockDivider( BCM2835_SPI_CLOCK_DIVIDER_16 );
	bcm2835_spi_setChipSelectPolarity( BCM2835_SPI_CS0, LOW );

	this->initialized = true;
}



void Serial2Wifi_SPI_Client::initialize(int baudRate)
{
	this->initializeQuietly();
	this->sendInitData();

	this->initialized = true;
}



void Serial2Wifi_SPI_Client::sendInitData()
{
	if (this->initialized == false)
		throw "Serial2Wifi SPI Client is uninitialized";

	char send_data[128];
	char read_data[128];

	for( int i = 0; i < 128; i++ )
		send_data[i] = '\n';

	int timeout = 0;
	while( !this->data_ready() )
	{
		++timeout;
		if( timeout > 20000000 )
			throw "device not ready";
	}

	bcm2835_gpio_clr( this->manual_cs );
	bcm2835_spi_transfernb( send_data, read_data, 6 );
	bcm2835_gpio_set( this->manual_cs );	
}



void Serial2Wifi_SPI_Client::close()
{
	if (this->initialized == false)
		throw "Serial2Wifi SPI Client is uninitialized";

	bcm2835_spi_end();
	bcm2835_close();

	this->initialized = false;
	this->first_run   = true;
}



int Serial2Wifi_SPI_Client::getBaudRate()
{
	if (this->initialized == false)
		throw "Serial2Wifi SPI Client is uninitialized";

	return(0);
}

void Serial2Wifi_SPI_Client::pulseCS()
{
	while( true )
	{
		printf("Clearing CS	MANUAL_SPI_CS_1 = %d\n",MANUAL_SPI_CS_1);
		bcm2835_gpio_clr( this->manual_cs );
		printf("Setting CS	this->manual_cs = %d\n",this->manual_cs);
		bcm2835_gpio_set( this->manual_cs );
	}
}

int Serial2Wifi_SPI_Client::sendCommandBlocking(const char* command, char* buffer)
{
	if (this->initialized == false)
		throw "Serial2Wifi SPI Client is uninitialized";

	try
	{
		this->sendData( command, strlen(command) );
	}
	catch (const char *e)
	{
		printf("%s\n", e);
		return(-1);
	}

	try
	{
		this->recvCommandResult(buffer);
	}
	catch (const char *e)
	{
		printf("%s\n", e);
		return(-2);
	}


	int len = strlen(buffer);

	if( buffer[len-1] == 21 )
		len = len - 1;

	if( buffer[len-6] == 79 && buffer[len-5] == 75 && buffer[len-4] == 13 && buffer[len-3] == 10 && buffer[len-2] == 62 && buffer[len-1] == 32 )
		return(1);
	else
		return(0);
}



int Serial2Wifi_SPI_Client::sendCommandNonBlocking(const char* command)
{
	if (this->initialized == false)
		throw "Serial2Wifi SPI Client is uninitialized";

	try
	{
		this->sendData( command, strlen(command) );
	}
	catch (const char *e)
	{
		printf("%s\n", e);
		return(-1);
	}

	return(1);
}



int Serial2Wifi_SPI_Client::sendDataBlocking(char *txBuffer, int txSize, char *rxBuffer, int rxSize)
{
	if (this->initialized == false)
		throw "Serial2Wifi SPI Client is uninitialized";

	try {
		this->sendData( txBuffer, txSize );
	}
	catch (const char *e) {
		printf("%s\n", e);
		return(0);
	}
	
	int dwBytesRead;
	try {
		dwBytesRead = this->recvData(rxBuffer, rxSize);
	}
	catch (const char *e) {
		printf("rxSize = %d\n", rxSize);
		printf("%s\n", e);
		return(0);
	}

	return(dwBytesRead);
}



int Serial2Wifi_SPI_Client::sendDataNonBlocking(char *txBuffer, int txSize)
{
	if (this->initialized == false)
		throw "Serial2Wifi SPI Client is uninitialized";

	try {
		this->sendData( txBuffer, txSize );
	}
	catch (const char *e) {
		printf("%s\n", e);
		return(0);
	}

	return(1);
}



int Serial2Wifi_SPI_Client::sendData(const char* txBuffer, int txSize)
{
	char sendBuffer[1500];
	char readBuffer[1500];

	int  sendSize = txSize;

	if( txSize%2 == 1 )
		sendSize++;

	for( int i = 0; i < txSize; i += 2 )
	{
		sendBuffer[i] = txBuffer[i+1];
		sendBuffer[i+1] = txBuffer[i];

		if( i+1 == txSize )
			sendBuffer[i] = '\n';
	}

	int timeout = 0;
	while( !this->data_ready() )
	{
		++timeout;
		if( timeout > 20000000 )
		{
			throw "write error: data ready pin is low";
		}
	}

	bcm2835_gpio_clr( this->manual_cs );
	bcm2835_spi_transfernb( sendBuffer, readBuffer, sendSize );

	timeout = 0;
	while( this->data_ready() )
	{
		bcm2835_gpio_set( this->manual_cs );
		++timeout;
		if( timeout > 20000000 )
		{
			throw "write error: device not ready";
		}
	}

	return(1);
}



int Serial2Wifi_SPI_Client::recvCommandResult(char *rxBuffer)
{
	char readBuffer[10000];

	char send_data[128];

	int timeout = 0;
	while( !this->data_ready() )
	{
		++timeout;
		if( timeout > 200000000 )
			throw "read error: device not ready";
	}

	char *read_ptr = readBuffer;

	bcm2835_gpio_clr( this->manual_cs );
	while( this->data_ready() )
	{
		bcm2835_spi_transfernb( send_data, read_ptr, 1 );
		read_ptr++;
	}
	bcm2835_gpio_set( this->manual_cs );

	readBuffer[read_ptr-readBuffer] = 0;

	if( rxBuffer != 0 )
	{
		int how_many = read_ptr - readBuffer;

		if( how_many%2 == 1 )
			how_many--;

		for( int i = 0; i < how_many; i+= 2 )
		{
			rxBuffer[i]   = readBuffer[i+1];
			rxBuffer[i+1] = readBuffer[i];
			rxBuffer[i+2] = '\0';
		}
	}

	return(read_ptr - readBuffer);
}



int Serial2Wifi_SPI_Client::recvData(char *rxBuffer, int rxSize)
{
	char  readBuffer[10000] = { '\0' };
	char  outputBuffer[10000] = { '\0' };
	int   outputTail = 0;

	clock_t begin, end;

	int done = 0;

	int dwBytesRead;
	int dwBytesToRead = rxSize;


	done = 0;
	begin = clock();
	while (done == 0)
	{
//		ReadFile(this->serialPort, rxBuffer + outputTail, dwBytesToRead, &dwBytesRead, NULL);

		if (dwBytesRead != 0)
		{
			outputTail = outputTail + dwBytesRead;

			printf("dwBytesRead = %d, outputTail = %d\n", dwBytesRead, outputTail);


			if (outputTail == rxSize)
				return(outputTail);
		}
		else
		{
			end = clock();

			if ((double)(end - begin) / CLOCKS_PER_SEC > this->readTimeout)
				throw "read timeout";
		}
	}

	return(outputTail);
}



double Serial2Wifi_SPI_Client::setReadTimeout(double timeout)
{
	if (timeout < 0)
		throw "Invalid timeout value";

	this->readTimeout = timeout;
	return(this->readTimeout);
}
double Serial2Wifi_SPI_Client::getReadTimeout()
{
	return(this->readTimeout);
}

double Serial2Wifi_SPI_Client::setWriteTimeout(double timeout)
{
	if (timeout < 0)
		throw "Invalid timeout value";

	this->writeTimeout = timeout;
	return(this->writeTimeout);
}
double Serial2Wifi_SPI_Client::getWriteTimeout()
{
	return(this->writeTimeout);
}



int Serial2Wifi_SPI_Client::data_ready()
{
	return( 0 != bcm2835_gpio_lev( this->data_ready_pin ) );
}

void Serial2Wifi_SPI_Client::print_reply( char* read_ptr, int how_many )
{
	unsigned char write_ptr[256]; 

	if( how_many%2 == 1 )
		how_many--;

	for( int i = 0; i < how_many; i += 2 )
	{
		write_ptr[i]   = read_ptr[i+1];
		write_ptr[i+1] = read_ptr[i];
		write_ptr[i+2] = '\0';
	}

	printf("%s\n",write_ptr);
}



